﻿Imports System.Collections.ObjectModel
Imports BindigTest.ViewModel.Infrastructure

Namespace ViewModel
    Public Class PersonsViewModel
        Inherits ViewModelBase

        Private _dm As IPersonDataManager

        Public Sub New()
            Me.New(New PersonDataManager)
        End Sub

        Public Sub New(dm As IPersonDataManager)
            _dm = dm
            If Reload.CanExecute(Nothing) Then Reload.Execute(Nothing)
        End Sub


        Private _allPersons As ObservableCollection(Of PersonViewModel)
        Public Property AllPersons() As ObservableCollection(Of PersonViewModel)
            Get
                Return _allPersons
            End Get
            Set(ByVal value As ObservableCollection(Of PersonViewModel))
                _allPersons = value
                RaisePropertyChanged()
            End Set
        End Property

        Private _currentPerson As PersonViewModel
        Public Property CurrentPerson() As PersonViewModel
            Get
                Return _currentPerson
            End Get
            Set(ByVal value As PersonViewModel)
                _currentPerson = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _includeDeleted As Boolean
        Public Property IncludeDeleted() As Boolean
            Get
                Return _includeDeleted
            End Get
            Set(ByVal value As Boolean)
                _includeDeleted = value
                RaisePropertyChanged()
                If Reload.CanExecute(Nothing) Then Reload.Execute(Nothing)
            End Set
        End Property


        Private _reload As ICommand
        Public ReadOnly Property Reload As ICommand
            Get
                If _reload Is Nothing Then _reload = New RelayCommand(AddressOf Reload_Execute)
                Return _reload
            End Get
        End Property

        Private Sub Reload_Execute(obj As Object)
            Dim reload As Boolean = False
            If obj IsNot Nothing Then
                reload = Boolean.Parse(obj)
            End If
            AllPersons = New ObservableCollection(Of PersonViewModel)()

            Dim allModelPersons = _dm.GetAviablePersons(IncludeDeleted, reload).ToList() 'Die Modelobjekte holen
            'Das sind aber Model.Person keine PersonViewModel. Also machen wir für jedes Model ein ViewModel
            allModelPersons.ForEach(Sub(x) AllPersons.Add(New PersonViewModel(x))) 'Dank dem zweiten Konstruktor ist das kein Thema und recht einfach
            CurrentPerson = AllPersons.FirstOrDefault()
        End Sub

        Private _save As ICommand
        Public ReadOnly Property Save As ICommand
            Get
                If _save Is Nothing Then _save = New RelayCommand(AddressOf Save_Execute)
                Return _save
            End Get
        End Property

        Private Sub Save_Execute(obj As Object)
            If Not _dm.SaveChanges(CurrentPerson.modelObject) Then
                'Meldung ausgeben oder sonstwas
                'in diesem Fall eine Exception
                Throw New Exception("Fehler beim Speichern der aktuellen Person")
            End If
        End Sub

        Private _goNext As ICommand
        Public ReadOnly Property GoNext As ICommand
            Get
                If _goNext Is Nothing Then _goNext = New RelayCommand(AddressOf GoNext_Execute, AddressOf GoNext_CanExecute)
                Return _goNext
            End Get
        End Property

        Private Function GoNext_CanExecute(obj As Object) As Boolean
            If AllPersons Is Nothing Or CurrentPerson Is Nothing Then Return False
            Dim currIndex = AllPersons.IndexOf(CurrentPerson)
            Return currIndex < AllPersons.Count - 1
        End Function

        Private Sub GoNext_Execute(obj As Object)
            Dim currIndex = AllPersons.IndexOf(CurrentPerson)
            CurrentPerson = AllPersons(currIndex + 1)
        End Sub

        Private _goPrevious As ICommand
        Public ReadOnly Property GoPrevious As ICommand
            Get
                If _goPrevious Is Nothing Then _goPrevious = New RelayCommand(AddressOf GoPrevious_Execute, AddressOf GoPrevious_CanExecute)
                Return _goPrevious
            End Get
        End Property

        Private Function GoPrevious_CanExecute(obj As Object) As Boolean
            If AllPersons Is Nothing Or CurrentPerson Is Nothing Then Return False
            Dim currIndex = AllPersons.IndexOf(CurrentPerson)
            Return currIndex > 0
        End Function

        Private Sub GoPrevious_Execute(obj As Object)
            Dim currIndex = AllPersons.IndexOf(CurrentPerson)
            CurrentPerson = AllPersons(currIndex - 1)
        End Sub
    End Class
End Namespace